#include <sys/stat.h>
#include <sys/wait.h>
#include "np.h"

#define MAXFD      64
#define MAXSOCKADDR  128
#define TRANSFER_BUFFER_SIZE     10000
#define LISTENQ	1024

/*
 * argv[1] = destination server;
 * argv[2] = desintation port;
 */

void proxy (int fd, char *szHost, const char *szPort);
void *TransferThread (void *data);
void sig_chld(int signo);

typedef struct tag_transfer {
    int fdSource;
    int fdDest;
} Transfer;

extern int daemon_proc;

/*
 * int main()
 */
int main (int argc, char *argv[])
{
  int connfd;
  socklen_t clilen;
  struct sockaddr *cliaddr;

  /* Run proxy from inetd -SP 1/22/99 */
  daemon_inetd(argv[0],0);

  if ( (cliaddr = malloc(MAXSOCKADDR)) == NULL) {
    perror("malloc error");
    exit(1);
  }
  clilen = MAXSOCKADDR;
  if (getpeername(0, cliaddr, &clilen) < 0) {
    perror("getpeername error");
    exit (1);
  }
  syslog(LOG_INFO, "connection from %s", Sock_ntop(cliaddr, clilen));
  proxy (connfd, argv[1], argv[2]);
  close (0);
  exit (0);
}

/*
 * void proxy()
 *
 * Description:
 * Proxy function.  Proxies all data from file descriptor
 * to the specified host.  The file descriptor is assumed
 * to be socket opened with any socket tools.
 */
void proxy (int fdIn, char *szHost, const char *szPort)
{
  int fdOut;
  pthread_t ptWrite;
  Transfer tRead, tWrite;

  fdOut = tcp_connect (szHost, szPort);

  /* Split to process incomming traffic and outgoing traffic... */
  /* One thread for each train of traffic.                      */

  /* Create a thread for the Writing to our Out server.         */
  tWrite.fdSource = fdIn;
  tWrite.fdDest = fdOut;
  pthread_create (&ptWrite, NULL, TransferThread, &tWrite);

  tRead.fdSource = fdOut;
  tRead.fdDest = fdIn;
  TransferThread(&tRead);
}

/*
 * void *TransferThead()
 * 
 * Description:
 * This function actually performs the data transfer
 * between two socket descriptors. It should be passed 
 * the Transfer structure.  fdSource and fdDest will the 
 * be source and destination file descriptors.  This 
 * function just loops reading data from fdSource and 
 * writing it to fdDest.
 */
void * TransferThread (void *data)
{
  Transfer *tControl;
  char szBuffer[TRANSFER_BUFFER_SIZE];
  int nReadAmt, nWriteAmt;

  tControl = data;

  for (;;) {
     nReadAmt = read (tControl->fdSource, &szBuffer, TRANSFER_BUFFER_SIZE);

     if (nReadAmt <= 0)  {
           close(tControl->fdDest);
           close(tControl->fdSource);
           return NULL; 
      }
            
      while (nReadAmt > 0)
	{
	  nWriteAmt = write (tControl->fdDest, &szBuffer, nReadAmt);
          
	  if (nWriteAmt <= 0 )
	    return NULL;

	  nReadAmt -= nReadAmt;
	}
    }

  return data;
}

/*
 * Function for running proxy from inetd (super server)
 */
void
daemon_inetd(const char *pname, int facility)
{
  int daemon_proc;

  daemon_proc = 1;
  openlog(pname, LOG_PID, facility);
}
